##dyn.load("pcd1.so")
library(MASS)
library(Matrix)


##Vectorization
mstack <- function(S){
  stack_S <- vector()
  m <- nrow(S)
  n <- ncol(S)
  stack_S <- matrix(ncol=1,nrow=m*n)
  for(f in 1:m){
      stack_S[(1+n*(f-1)):(n*f),1] <- as.numeric(S[f,])
  }
  stack_S
}


##Computer l1 or l2 norm
lnorm <- function(a,l=1) (sum(abs(a)^l))^(1/l)

###Reduced-rank regression
RRR <- function(Y,X,nrank=1,ypy.svd=TRUE,c.svd=TRUE){

   q <- ncol(Y)
   n <- nrow(Y)
   p <- ncol(X)
   S_yx <- t(Y)%*%X
   S_xx <- t(X)%*%X

   ##S_xx_inv <- tryCatch(solve(S_xx+0.1*diag(p)),error=function(e)ginv(S_xx))
   S_xx_inv <- ginv(S_xx)
   C_ls <- S_xx_inv%*%t(S_yx)
   
   if(ypy.svd){
     XC <- X%*%C_ls
     ##svdXC <- svd(XC,nu=nrank,nv=nrank)
     ##SVD may not converge
     svdXC <- tryCatch(svd(XC,nu=nrank,nv=nrank),error=function(e)2)
     if(tryCatch(svdXC==2,error=function(e)3)==3){
        A <- svdXC$v[,1:nrank]
        Ad <- (svdXC$d[1:nrank])^2
     }else{
       ypy.svd <- FALSE
     }
   }
   if(!ypy.svd){
     SS <- S_yx%*%C_ls
     SS <- (SS+t(SS))/2
     eigenSS <- eigen(SS,symmetric=TRUE)
     A <- as.matrix(eigenSS$vectors[,1:nrank])
     Ad <- eigenSS$values[1:nrank]   
   }
      
   AA <- A%*%t(A)
   C_rr <-C_ls%*%AA
   
   if(c.svd){
     svd_C <- svd(C_rr,nv=nrank,nu=nrank)
     U <- as.matrix(svd_C$u[,1:nrank])
     V <- as.matrix(svd_C$v[,1:nrank])
     D <- diag(svd_C$d[1:nrank],nrow=nrank)

     ##return ls estimator C_ls, reduced-rank estimator C_rr
     ##return SVD of C_rr
     list(A=A,Ad=Ad,C_ls=C_ls,C_rr=C_rr,U=U,V=V,D=D,C=C_rr,rank=nrank)
   }else{
     list(A=A,Ad=Ad,C_ls=C_ls,C_rr=C_rr,C=C_rr,rank=nrank)        
   }
}







###Initialization of EEA 
SRRR_ini<-function(Y,X,U0=NULL,V0=NULL,D0=NULL,Uw=U0,Vw=V0,nrank=1,adaptive=TRUE,gamma0=2,tol=1e-8){ 

   #require(lars)
   p <- ncol(X)
   q <- ncol(Y)
   n <- nrow(Y)
   
   if(is.null(U0) || is.null(V0) || is.null(D0)){ 
     ##Calculate LS estimator and RR estimator   
     ini <- RRR(Y,X,nrank=nrank,c.svd=TRUE)
     C_ls <- ini$C_ls
     C_rr <- ini$C         
     U0 <- ini$U
     V0 <- ini$V
     D0 <- ini$D     
   }else{
     D0 <- diag(D0,nrow=nrank)         
     C_rr <- U0%*%D0%*%t(V0)
     C_ls <- NULL
   }
      
   if(is.null(Uw) || is.null(Vw)){
     Uw <- U0
     Vw <- V0 
   }     
   
   ###Compute exclusive layers
   Yel <- array(dim=c(n,q,nrank))
   Wuel <- matrix(nrow=p,ncol=nrank,1)   
   Wvel <- matrix(nrow=q,ncol=nrank,1)

   ###Initial nonzero singular vectors
   Uini <- matrix(nrow=p,ncol=nrank,0)
   Vini <- matrix(nrow=q,ncol=nrank,0) 

   ###Maximum lambda
   lam_max <- vector()
   
   for(k in 1:nrank){
     Ck <- U0[,k]%*%t(V0[,k])*D0[k,k] 
     Cmk <- C_rr - Ck   
     Yel[,,k] <- Y - X%*%Cmk

     if(adaptive==TRUE) {
       if(sum(Uw[,k]==0)==p) Wuel[,k] <- 1
       else{
         Wuel[,k] <- abs(Uw[,k])^gamma0
         Wuel[,k][Wuel[,k]==0] <- tol
       }  
       if(sum(Vw[,k]==0)==q) Wvel[,k] <- 1       
       else {
         Wvel[,k] <- abs(Vw[,k])^gamma0      
         Wvel[,k][Wvel[,k]==0] <- tol
       }
       ##Wdel[k] <- ifelse(Dw[k]==0,tol,abs(Vw[,k])^gamma0)   
         
     }
     
     lam <- matrix(nrow=p,ncol=q,0)
     for(a in 1:p){
       for(b in 1:q){
         lam[a,b] <- abs(Wuel[a,k]*Wvel[b,k]*t(X[,a])%*%Yel[,b,k])
       }
     }
     
     max_id <- which.max(lam)
     b <- round(max_id/p-0.50001)+1
     a <- max_id-(b-1)*p
     lam_max[k] <- lam[a,b]
     Uini[as.integer(a),k] <- 1     
     Vini[as.integer(b),k] <- 1      
   } 
   
   list(Yel=Yel,C_ls=C_ls,C_rr=C_rr,
        Wuel=Wuel,Wvel=Wvel,
        U0=U0,V0=V0,D0=D0,
        Uini=Uini,Vini=Vini,
        lam_max=lam_max,Mdim=c(p,q,n)
   )

}


##Sparse unit-rank regression for a fixed lambda
##Have to input initial values U,V and weight matrices WU,WV, otherwise computed from RRR
SURR<-function(Y,X,lambda,U=NULL,V=NULL,WU=NULL,WV=NULL,Xtran=NULL,
                   max.iter=100,converge=1e-4,inner.conv=1e-4,inner.iter=50,
                   adaptive=TRUE,gamma0=2){ 	

   p <- ncol(X)
   q <- ncol(Y)
   n <- nrow(Y)

   j <- 1 
   conv<-vector() 
   conv[1]<-converge*2	

   if(is.null(U) | is.null(V) | is.null(WU) | is.null(WV)){
      ini <- SRRR_ini(Y,X,nrank=1,adaptive=adaptive,gamma0=gamma0)
      U <- ini$U0
      V <- ini$V0    
      WU <- diag(as.vector(ini$Wuel))
      WV <- diag(as.vector(ini$Wvel))
   }
   
   ##Transform X by weight matrix
   if(is.null(Xtran)) Xtran <- X%*%WU
   Yvec <- as.vector(Y)

   while(j<=max.iter && conv[j]>=converge && sum(U==0)!=p){

      U_p <- as.matrix(U)
      V_p <- as.matrix(V)

      ##U-step, adaptive lasso
      ##X_U <- kronecker(WV%*%V,Xtran)
      X_U <- .Call("kronecker",WV%*%V,Xtran)
      U <- as.matrix(.Call("pcd", X_U, Yvec, lambda, as.vector(U_p), inner.conv, as.integer(inner.iter)))      
      D <- lnorm(U)
      U <- U/D      
      
      ##V-step, adaptive lasso
      ##This step can be made explicit
      ##X_V <- kronecker(WV,Xtran%*%U)
      X_V <- .Call("kronecker",WV,Xtran%*%U) 
      V <- as.matrix(.Call("pcd", X_V, Yvec, lambda, as.vector(V_p), inner.conv, as.integer(inner.iter))) 
      D <- lnorm(V)
      V <- V/D    

      ##Convergence criterion
      conv[j+1] <- lnorm(U%*%t(V)-U_p%*%t(V_p),2)/lnorm(U_p%*%t(V_p),2)
      j<-j+1   
   }


   residual <- (Yvec-X_V%*%V*D)    
   sse <- t(residual)%*%residual
   obj <- sse/2/q/n + lambda*D
  
   if(conv[j]<=converge) conv_flag <- TRUE
   else conv_flag <- FALSE 
    
   U <- round(U,8)
   V <- round(V,8)
   dfu0 <- sum(U!=0)
   dfv0 <- sum(V!=0)

   V <- WV%*%V*D   
   U <- WU%*%U
   DV<- lnorm(V,2)
   DU <- lnorm(U,2)
   V <- V/DV
   U <- U/DU
   D <- DV*DU

   BIC <-  log(sse) + log(q*n)/q/n*(dfv0+dfu0-1)
   BICP <- log(sse) + 2*log(p*q)/q/n*(dfv0+dfu0-1)
   AIC <-  log(sse) + 2/q/n*(dfv0+dfu0-1)   

   list(obj=obj,
        conv=conv[max(1,j-50):j],
        iter=j-1,sse=sse,
        df=c(dfu0,dfv0),
        IC=c(BIC,BICP,AIC),	
        U=U,V=V,D=D,conv_flag = conv_flag)
	   
}





###SRRR, exclusive extration algorithm
###Compute the solution path of each layer
SRRR_eea <- function(Y,X,nrank,adaptive=TRUE,gamma0=2,
                     ini=SRRR_ini(Y,X,nrank=nrank,adaptive=adaptive,gamma0=gamma0),
                     nlam=100,minlam=1e-5,inner.iter=100,inner.conv=1e-4,pcd.iter=50,pcd.conv=1e-4,IC=1){

  p <- ncol(X)
  q <- ncol(Y)
  n <- nrow(Y)

  ###Information criteria  
  ##BIC, BICP, AIC
  ##Default is BIC
  IC_path <- array(dim=c(nlam,nrank,3),NA)   

  ###Solution path
  U_path <- array(dim=c(p,nlam,nrank),NA)
  V_path <- array(dim=c(q,nlam,nrank),NA)
  D_path <- matrix(nrow=nlam,ncol=nrank,NA)

  ###Final solution selected by IC
  U <- matrix(nrow=p,ncol=nrank)
  V <- matrix(nrow=q,ncol=nrank)  
  D <- rep(NA,nrank)
  Lam <- rep(NA,nrank)      
  

  for(k in 1:nrank){
    
    lambda <- c(exp(seq(log(ini$lam_max[k]*0.95),log(ini$lam_max[k]*minlam),length=nlam-1)),0)

    UU <- matrix(ncol=nlam,nrow=p)
    VV <- matrix(ncol=nlam,nrow=q)
    DD <- rep(NA,nlam)
    CONV <- vector()
    SSE <- vector()

    U_star <- NULL
    OK <- 1  
    Repeat <- FALSE

    Uini <- ini$Uini[,k]
    Vini <- ini$Vini[,k]    
    WU <- diag(as.vector(ini$Wuel[,k]))
    WV <- diag(as.vector(ini$Wvel[,k]))
    Yel <- ini$Yel[,,k]
    Xtran <- X%*%WU
    
    ##fitting
    i <- 1
    while(i <= nlam){
      fit <-  tryCatch(SURR(Yel,X,lambda=lambda[i],U=Uini,V=Vini,WU=WU,WV=WV,Xtran,
                                max.iter=inner.iter,converge=inner.conv,inner.iter=pcd.iter,inner.conv=pcd.conv),
                     error=function(e)2)         
      if(tryCatch(fit==2,error=function(e)3)==3){
        ###If converges       
        if(fit$conv_flag==TRUE){
          CONV[i] <- fit$conv_flag
          ##BIC[i,k] <- fit$IC[1]
          ##BICP[i,k] <- fit$IC[2]          
          ##AIC[i,k] <- fit$IC[3]
          IC_path[i,k,] <- fit$IC
          
          SSE[i] <- fit$sse
          
          VV[,i] <- fit$V
          UU[,i] <- fit$U
          DD[i] <- fit$D
          Uini <- fit$U
          Vini <- fit$V
          
          U_star <- fit$U

          OK <- 1
          Repeat <- FALSE
        }else Repeat <- TRUE
      }
      
      ###If fail to converge (rarely happens), try different initial values.
      ###Modify or delete this later
      if(tryCatch(fit==2,error=function(e)3)==1 || Repeat==TRUE) {
        print(paste("Fail to converge"))     
        CONV[i] <- FALSE
        OK <- OK + 1
        if(OK>=3 & OK<6) Uini[Uini!=0] <- Uini[Uini!=0] + rnorm(sum(Uini!=0),0,1)
        if(OK>=6) Uini <- NULL
        if(OK==8) Uini <- U_star
        if(OK<8){i <- i-1}else OK <- 1
      } 
      i <- i+1
      ###if(round((i-1)/100)==(i-1)/100 & k==nrank){
      ###  print(paste("CONV=",CONV[ifelse(OK>1,i,i-1)],"   i=",ifelse(OK>1,i,i-1),sep=""))
      ###}
    }

    logSY <- log(sum(Yel^2))
    ###choose lambda
    minid <- apply(IC_path[,k,],2,which.min)
   
    ##Final solution based on IC
    if(logSY<=IC_path[minid[IC],k,IC]){
        U[,k] <- 0
        V[,k] <- 0
        D[k] <- 0
        Lam[k] <- ini$lam_max[k]
    }else{
        U[,k] <- UU[,minid[IC]]
        V[,k] <- VV[,minid[IC]]
        D[k] <- DD[minid[IC]]
        Lam[k] <- lambda[minid[IC]]
    }
              
    U_path[,,k] <- UU
    V_path[,,k] <- VV
    D_path[,k] <- DD  
  }


  ###The current solution is U,D,V
  ###Update singular values
  nzid <- which(D!=0)
  h <- length(nzid)
  if(h!=0){
    Xs <- matrix(ncol=h,nrow=n*q,NA)
    fac <- vector()
    df <- vector()
    for(k in 1:h){
        Ck <- U[,nzid[k]]%*%t(V[,nzid[k]])
        fac[k] <- sum(abs(Ck)/(ini$Wuel[,nzid[k]]%*%t(ini$Wvel[,nzid[k]])))*Lam[nzid[k]]
        Xs[,k] <- mstack(X%*%Ck)/fac[k] 
        df[k] <- sum(U[,nzid[k]]!=0) + sum(V[,nzid[k]]!=0)-1
        ##df[k] <- p+q-1
    } 
    Ys <- mstack(Y)
    D[nzid] <- as.matrix(.Call("pcd", Xs, Ys, 1, as.vector(D[nzid]), inner.conv, as.integer(inner.iter)))/fac

  }
  ###Update U,D,V
  U[,which(D==0)] <- 0
  V[,which(D==0)] <- 0
  
  list(Upath=U_path,Vpath=V_path,Dpath=D_path,IC_path=IC_path,U=U,V=V,D=D)

}




###An alternating lasso algorithm
SRRR_alt <- function(Y,X,nrank=3,nlam=100,minlam=1e-6,maxlam=0.80,IC=1,
                conv=1e-4,iter=100,inner.conv=1e-3,inner.iter=20,wgamma=2){

   #require(lars)
   p <- ncol(X)
   q <- ncol(Y)
   n <- nrow(Y)

   ############Compute weights and lam_max#############
   ini <- RRR(Y,X,nrank=nrank,c.svd=TRUE)        
   Wu <- as.matrix(ini$U^{wgamma})
   Wv <- as.matrix(ini$V^{wgamma})
   Wd <- as.vector(diag(ini$D^{wgamma}))
   Wun <- as.matrix(ini$U^{-wgamma})
   Wvn <- as.matrix(ini$V^{-wgamma})
   Wdn <- as.vector(diag(ini$D^{-wgamma}))
   
   ##initial RRR solution 
   U0 <- ini$U
   V0 <- ini$V
   D0 <- diag(ini$D)       
   
   #############Maximum lambda######################
   ###Based on rank-1 LS estimator
   ##inir1 <- RRR(Y,X,nrank=1,c.svd=TRUE)
   ##C_lsr1 <- inir1$C_ls
   ##U0r1 <- inir1$U
   ##V0r1 <- inir1$V
   ##D0r1 <- inir1$D        

   lam <- matrix(nrow=p,ncol=q,0)
   for(a in 1:p){
       for(b in 1:q){
         lam[a,b] <- abs((Wu[a,1]*Wv[b,1]*Wd[1])*t(X[,a])%*%Y[,b])
       }
   }
     
   max_id <- which.max(lam)
   b <- round(max_id/p-0.50001)+1
   a <- max_id-(b-1)*p
   lam_max <- lam[a,b]
     
   ###Initial nonzero solution###
   ###Not used
   Uini <- matrix(nrow=p,ncol=nrank,0)
   Vini <- matrix(nrow=q,ncol=nrank,0)    
   Uini[as.integer(a),1] <- ifelse(t(X[,a])%*%Y[,b]>0,1,-1)     
   Vini[as.integer(b),1] <- 1      
  
  
  
   ############objective function for adaptive lasso
   OBJ <- function(Y,X,U,D,V,lambda,Wu,Wv,Wd) {      
      pk <- vector()
      for(k in 1:ncol(U)){
        pk[k] <- lnorm(U[,k]/Wu[,k],1)*lnorm(V[,k]/Wv[,k],1)*lnorm(D[k]/Wd[k],1)
      } 
      lnorm(Y-X%*%U%*%diag(as.vector(D),ncol=ncol(U))%*%t(V),2)^2/2+lambda*sum(pk)
   }  

  
   ###Lambda sequence, From large to small
   lamseq <- c(0,exp(seq(log(lam_max*minlam),log(lam_max*maxlam),length=nlam-1)))
  
   ######Vectorize Y
   Yvec <- as.matrix(as.vector(Y))
  
   ###Store solution path 
   U_path <- array(dim=c(p,nrank,nlam),0)    
   V_path <- array(dim=c(q,nrank,nlam),0)  
   D_path <- matrix(nrow=nrank,ncol=nlam,0)
   Obj_path <- matrix(nrow=iter+1,ncol=nlam,NA)
   
   ##Store Rank
   Rank_path <- rep(0,nlam)
   
   ###Store information critera
   IC_path <- matrix(nrow=4,ncol=nlam)   

   U_path[,,1] <- U0
   V_path[,,1] <- V0
   D_path[,1] <- D0 

   ###################Fitting solution path 
   for(kk in 1:nlam){ 

      lambda <- lamseq[kk]

      U <- U_path[,,max(1,kk-1)]
      V <- V_path[,,max(1,kk-1)]
      D <- D_path[,max(1,kk-1)]  

      ##Remove zero columns
      zeroid <- which(apply(U,2,sum)*apply(V,2,sum)*D==0)
      lzero <- length(zeroid)
      if(lzero!=0){
        U <- as.matrix(U[,-zeroid])
        V <- as.matrix(V[,-zeroid])
        D <- as.matrix(D[-zeroid])  
      }

      j <- 1
      diff <- vector()
      diff[1] <- conv*2
  
      ##Initial Obj
      if(lzero!=0){
        Obj_path[1,kk] <- OBJ(Y,X,U,D,V,lambda,as.matrix(Wu[,-zeroid]),
                                as.matrix(Wv[,-zeroid]),as.vector(Wd[-zeroid]))
      }else{
        Obj_path[1,kk] <- OBJ(Y,X,U,D,V,lambda,Wu,Wv,Wd)
      }
      
      ###Fit alternating lasso 
      breakFlag <- FALSE  
      while(j<=iter && diff[j] >= conv){
      
        U_p <- U
        V_p <- V
        D_p <- D
    
        ###########################
        ##Xu <- kronecker(V%*%D,X) 
        ##system.time(sweep(Xu,MARGIN=2,Wx,`*`))
        ##system.time(t(t(Xu)*Wx))
        ##system.time(.Call("matcolMply",Xu,Wx))
    

        ###############UD-lasso step######################
        if(lzero!=0){
            Wu0 <- Wun[,-zeroid]%*%diag(Wdn[-zeroid]*diag(t(Wvn[,-zeroid])%*%abs(V)),ncol=nrank-length(zeroid))    
        }else{
            Wu0 <- Wun%*%diag(Wdn*diag(t(Wvn)%*%abs(V)),ncol=nrank)    
        }
        Xu <- .Call("kronecker",V,X)    
        Xuw <- Xu%*%diag(as.vector(Wu0^(-1)))
        U <- matrix(nrow=p,ncol=nrank-length(zeroid),byrow=F, .Call("pcd", Xuw, Yvec, lambda, as.vector(U_p*Wu0), inner.conv, as.integer(inner.iter)))/Wu0
        

        #################################################
        ##Check zero ranks   
        zeroidj <- which(apply(U,2,sum)==0)
        if(length(zeroidj)!=0){
            ##ID of true zero layers
            if(length(zeroid)!=0){
                zeroid <- c(zeroid, (1:nrank)[-zeroid][zeroidj]) 
                lzero <- length(zeroid)      
            }else{
                zeroid <- c(zeroid, (1:nrank)[zeroidj])
                lzero <- length(zeroid)                             
            }
            if(lzero==nrank) breakFlag <- TRUE
            U <- as.matrix(U[,-zeroidj])
            V <- as.matrix(V[,-zeroidj])
            D <- as.matrix(D[-zeroidj])
        }
        if(breakFlag) break
        
        ##Normalize U    
        for(i in 1:ncol(U)) U[,i] <- U[,i]/lnorm(U[,i],2)
                   
                       
        ####################VD-lasso step########################
        if(length(zeroid)!=0){
            Wv0 <- Wvn[,-zeroid]%*%diag(Wdn[-zeroid]*diag(t(Wun[,-zeroid])%*%abs(U)),ncol=nrank-length(zeroid))
        }else{
            Wv0 <- Wvn%*%diag(Wdn*diag(t(Wun)%*%abs(U)),ncol=nrank-length(zeroid))
        }
        ##Use the Matrix package
        Xv <- as.matrix(kronecker(Diagonal(q),X%*%U))    
        Xvw <- Xv%*%diag(as.vector(t(Wv0^(-1))))    
        V <- matrix(nrow=q,ncol=nrank-length(zeroid),byrow=T, .Call("pcd", Xvw, Yvec, lambda, as.vector(t(V_p)), inner.conv, as.integer(inner.iter)))/Wv0

        #################################################
        ##Check zero ranks   
        zeroidj <- which(apply(V,2,sum)==0)
        if(length(zeroidj)!=0){
            ##ID of true zero layers
            if(length(zeroid)!=0){
                zeroid <- c(zeroid, (1:nrank)[-zeroid][zeroidj]) 
                lzero <- length(zeroid)      
            }else{
                zeroid <- c(zeroid, (1:nrank)[zeroidj])
                lzero <- length(zeroid)                             
            }
            if(lzero==nrank) breakFlag <- TRUE            
            U <- as.matrix(U[,-zeroidj])
            V <- as.matrix(V[,-zeroidj])
            D <- as.matrix(D[-zeroidj])
        }
        if(breakFlag) break
        
        ##Normalize V and get D   
        for(i in 1:ncol(V)){
            D[i] <- lnorm(V[,i],2)
            V[,i] <- V[,i]/D[i]
        }
    

        ###########################
        ##Compute objective function    
        j <- j + 1
        if(ncol(U_p)>ncol(U)){
            Udiff <- U_p
            Udiff[,1:ncol(U)] <- Udiff[,1:ncol(U)]-U
            Vdiff <- V_p
            Udiff[,1:ncol(V)] <- Udiff[,1:ncol(V)]-V        
            Ddiff <- D_p
            Udiff[1:ncol(V)] <- Ddiff[1:ncol(V)]-D      
            diff[j] <- lnorm(Udiff,2)/lnorm(U_p,2) + lnorm(Vdiff,2)/lnorm(V_p,2) + lnorm(Ddiff,2)/lnorm(D_p,2)
        }else{
            diff[j] <- lnorm(U-U_p,2)/lnorm(U_p,2) + lnorm(V-V_p,2)/lnorm(V_p,2) + lnorm(D-D_p,2)/lnorm(D_p,2)    
        }
    
        ##diff[j] <- lnorm(U-U_p,2)/lnorm(U_p,2) + lnorm(V-V_p,2)/lnorm(V_p,2)
        ##diff[j] <- lnorm(U%*%D%*%t(V)-U_p%*%D_p%*%t(V_p),2)/lnorm(U_p%*%D_p%*%t(V_p),2)

        if(lzero!=0){
            Obj_path[j,kk] <- OBJ(Y,X,U,D,V,lambda,as.matrix(Wu[,-zeroid]),
                                    as.matrix(Wv[,-zeroid]),as.vector(Wd[-zeroid]))
        }else{
            Obj_path[j,kk] <- OBJ(Y,X,U,D,V,lambda,Wu,Wv,Wd)
        }
    
        ##print(paste("iter=",j,"  diff=",U,U_p))
     }
     
     if(breakFlag) break
         
     if(lzero!=0){
        U_path[,-zeroid,kk] <- U
        V_path[,-zeroid,kk] <- V 
        D_path[-zeroid,kk] <- D                  
     }else{
        U_path[,,kk] <- U
        V_path[,,kk] <- V 
        D_path[,kk] <- D      
     } 

     ##Rank and df
     Rank_path[kk] <- nrank-lzero
     dfu0 <- sum(U!=0)
     dfv0 <- sum(V!=0)
     df <- dfu0 + dfv0 - Rank_path[kk]

     ####################################  
     ###Compute infomation criteria
     residual <- (Y-X%*%U_path[,,kk]%*%diag(D_path[,kk],ncol=nrank)%*%t(V_path[,,kk]))    
     sse <- sum(residual^2)
     ##BIC,BICP,AIC,GCV
     IC_path[1,kk] <-  log(sse) + log(q*n)/q/n*df
     IC_path[2,kk] <- log(sse) + 2*log(p*q)/q/n*df
     IC_path[3,kk] <-  log(sse) + 2/q/n*df
     IC_path[4,kk] <- sse/q/n/(1-df/q/n)^2   
   }
   
   
   
   ##Default is BIC
   ICid <- which.min(IC_path[IC,1:min(nlam,kk)])
   
   list(Obj_path=Obj_path,IC_path=IC_path,Rank_path=Rank_path,
        U_path=U_path,V_path=V_path,D_path=D_path,
        U=U_path[,,ICid],V=V_path[,,ICid],D=D_path[,ICid])

}

























###Generate initial values for EEA run
###U0,V0,D0 are initial values
###Uw and Vw are for constrcting weights
SRRR_orth_ini<-function(Y,X,U0=NULL,V0=NULL,D0=NULL,Uw=U0,Vw=V0,nrank=1,adaptive=TRUE,gamma0=2,tol=1e-8){ 

   p <- ncol(X)
   q <- ncol(Y)
   n <- nrow(Y)

   if(is.null(U0) || is.null(V0) || is.null(D0)){ 
     rrr <- RRR(Y,X,nrank)
     C_rr <- rrr$C_rr
     C_ls <- rrr$C_ls
     U0 <- rrr$U
     V0 <- rrr$V
     D0 <- rrr$D
   }else{
     D0 <- diag(D0,nrow=nrank) 
     C_rr <- U0%*%D0%*%t(V0)
     C_ls <- NULL
   }
      
   if(is.null(Uw) || is.null(Vw)){
     Uw <- U0
     Vw <- V0 
   }     
   
   ###Construct exclusive layers Yel
   ###Construct weights Wuel
   Yel <- array(dim=c(n,q,nrank))
   Wuel <- matrix(nrow=p,ncol=nrank,1)   
   Wvel <- matrix(nrow=q,ncol=nrank,1)
   Uini <- matrix(nrow=p,ncol=nrank,0)
   Vini <- matrix(nrow=q,ncol=nrank,0) 
   lam_max <- vector()
   
   for(k in 1:nrank){
     Ck <- U0[,k]%*%t(V0[,k])*D0[k,k] 
     Cmk <- C_rr - Ck   
     Yel[,,k] <- Y - X%*%Cmk

     if(adaptive==TRUE) {
       if(sum(Uw[,k]==0)==p | gamma0==0) Wuel[,k] <- 1
       else{
         Wuel[,k] <- abs(Uw[,k])^(-gamma0)
         Wuel[,k][Wuel[,k]==0] <- tol^(-1)
       }  
       if(sum(Vw[,k]==0)==q | gamma0==0) Wvel[,k] <- 1       
       else {
         Wvel[,k] <- abs(Vw[,k])^(-gamma0)      
         Wvel[,k][Wvel[,k]==0] <- tol^(-1)
       }  
     }
     
     lam <- matrix(nrow=p,ncol=q,0)
     for(a in 1:p){
       for(b in 1:q){
         lam[a,b] <- abs(t(X[,a])%*%Yel[,b,k]/Wuel[a,k]/Wvel[b,k])
       }
     }
     max_id <- which.max(lam)
     b <- round(max_id/p-0.50001)+1
     a <- max_id-(b-1)*p
     lam_max[k] <- lam[a,b]
     Uini[as.integer(a),k] <- 1     
     Vini[as.integer(b),k] <- 1      
   } 

   
   list(Yel=Yel,C_ls=C_ls,C_rr=C_rr,
        Wuel=Wuel,Wvel=Wvel,
        U0=U0,V0=V0,D0=D0,Uw=Uw,Vw=Vw,
        Uini=Uini,Vini=Vini,
        lam_max=lam_max,Mdim=c(p,q,n))

}



###SRRR for unit-rank model and a single lambda
SURR_orth <- function(Y,X,lambda=0,YX=NULL,Wu=NULL,Wv=NULL,Wd=1,
                      U0=NULL,V0=NULL,max.iter=100,converge=1e-4,
                      adaptive=TRUE,gamma0=2){
  
  if(is.null(YX)) YX <- t(Y)%*%X
   
   q <- ncol(Y)
   n <- nrow(Y)
   p <- ncol(X)   

   if(is.null(U0) | is.null(V0) | is.null(Wu) | is.null(Wv)){
      ini <- SRRR_orth_ini(Y,X,nrank=1,adaptive=adaptive,gamma0=gamma0)
      U0 <- ini$Uini
      V0 <- ini$Vini  
      Wu <- ini$Wuel
      Wv <- ini$Wvel
   }

  
  result <- .Call("srrr_orth",YX,Wv,Wu,Wd,V0,lambda,converge,as.integer(max.iter))
  V <- as.matrix(result[1:q])
  U <- as.matrix(result[(q+1):(p+q)])
  D <- result[p+q+1]
  
  ##calculate BIC
  residual <- Y-X%*%U%*%t(V)*D;
  sse <- sum(residual^2)
  logsse <- log(sse)
  U <- round(U,8)
  V <- round(V,8)
  dfu0 <- sum(U!=0)
  dfv0 <- sum(V!=0)
  dfmodel <- dfu0 + dfv0 -1
  BIC <- logsse + log(q*n)/q/n*dfmodel
  BICP <- logsse + 2*log(p*q)/q/n*dfmodel
  AIC <- logsse + 2/q/n*dfmodel


  list(sse=sse,dfu=dfu0,dfv=dfv0,
       IC=c(BIC,BICP,AIC),
	   U=U,V=V,D=D)	
}




##fit the EEA solution path
SRRR_orth_eea <- function(Y,X=diag(nrow(Y)),nrank=1,adaptive=TRUE,gamma0=2,
                     ini=SRRR_orth_ini(Y,X,nrank=nrank,adaptive=adaptive,gamma0=gamma0),
                     nlam=100,minlam=1e-6,inner.iter=100,inner.conv=1e-4,IC=1){

  p <- ncol(X)
  q <- ncol(Y)
  n <- nrow(Y)

  ##store IC
  IC_path <- array(dim=c(nlam,nrank,3),NA)

  ##store paths
  U_path <- array(dim=c(p,nlam,nrank),NA)
  V_path <- array(dim=c(q,nlam,nrank),NA)
  D_path <- matrix(nrow=nlam,ncol=nrank,NA)

  ##store results
  U <- matrix(nrow=p,ncol=nrank)
  V <- matrix(nrow=q,ncol=nrank)  
  D <- rep(NA,nrank)
  
  for(k in 1:nrank){
    
    lambda <- c(exp(seq(log(ini$lam_max[k]*0.97),log(minlam),length=nlam-1)),0)

    UU <- matrix(ncol=nlam,nrow=p)
    VV <- matrix(ncol=nlam,nrow=q)
    DD <- rep(NA,nlam)

    Uini <- ini$Uini[,k]
    Vini <- ini$Vini[,k]    
    Wu <- ini$Wuel[,k]
    Wv <- ini$Wvel[,k]
    Yel <- ini$Yel[,,k]
   
    ##fitting the path
    i <- 1
    while(i <= nlam){
      fit <-  SURR_orth(Yel,X,lambda=lambda[i],U0=Uini,V0=Vini,Wu=Wu,Wv=Wv,
                                 max.iter=inner.iter,converge=inner.conv)

      IC_path[i,k,] <- fit$IC          
      VV[,i] <- fit$V
      UU[,i] <- fit$U
      DD[i] <- fit$D
      Uini <- fit$U
      Vini <- fit$V
      
      i <- i + 1
    }

    logSY <- log(sum(Yel^2))
    minid <- apply(IC_path[,k,],2,which.min)
   
    ##Final solution based on IC
    if(logSY<=IC_path[minid[IC],k,IC]){
        U[,k] <- 0
        V[,k] <- 0
        D[k] <- 0
    }else{
        U[,k] <- UU[,minid[IC]]
        V[,k] <- VV[,minid[IC]]
        D[k] <- DD[minid[IC]]
    }

    U_path[,,k] <- UU
    V_path[,,k] <- VV
    D_path[,k] <- DD
  
  
  }


  list(Upath=U_path,Vpath=V_path,Dpath=D_path,IC_path=IC_path,U=U,V=V,D=D)

}





###IEEA for orthogonal model;
###This code only works for orthogonal design 
##Model: Y = XC + E
##nrank: rank of model to start, can be an upper bound of the true rank
##adaptive: whether using adaptive weights
SRRR_orth_ieea <- function(Y,X=diag(nrow(Y)),nrank=1,adaptive=TRUE,gamma0=2,nlam=100,minlam=1e-5,
                           iter=1,conv=1e-2, inner.iter=100,inner.conv=1e-4,IC=1){

   ini0 <- SRRR_orth_ini(Y,X,nrank=nrank,adaptive=adaptive,gamma0=gamma0)
   Uw <- ini0$Uw
   Vw <- ini0$Vw
   fit <- SRRR_orth_eea(Y,X,nrank,adaptive,gamma0,ini0,nlam,minlam,inner.iter,inner.conv,IC)
   
   d <- fit$D
   rnew <- sum(d!=0)
   
   dorder <- order(d,decreasing=TRUE)
   ##dorder <- 1:rnew
   U <- fit$U[,dorder[1:rnew]]
   V <- fit$V[,dorder[1:rnew]]
   D <- d[dorder[1:rnew]]
   Uw <- Uw[,dorder[1:rnew]]
   Vw <- Vw[,dorder[1:rnew]]
   
   niter <- 1
   diff <- 2*conv 
   null <- FALSE                          
   while(rnew>1 & niter < iter & diff >= conv) {
      U_p <- U
      V_p <- V
      D_p <- D
      rp <- rnew
      ini <- SRRR_orth_ini(Y,X,U0=U_p,V0=V_p,D0=D_p,Uw=Uw,Vw=Vw,nrank=rnew,adaptive=adaptive,gamma0=gamma0)
      fit <- SRRR_orth_eea(Y,X,rnew,adaptive,gamma0,ini,nlam,minlam,inner.iter,inner.conv,IC)
      d <- fit$D
      rnew <- sum(d!=0)
      if(rnew==0){
        U <- 0
        V <- 0
        D <- 0
        break
      }
      dorder <- order(d,decreasing=TRUE)
      ##dorder <- 1:rnew
      U <- fit$U[,dorder[1:rnew]]
      V <- fit$V[,dorder[1:rnew]]
      D <- d[dorder[1:rnew]]   
      Uw <- Uw[,dorder[1:rnew]]
      Vw <- Vw[,dorder[1:rnew]]  

      
      C_p <- U_p%*%diag(D_p,nrow=rp)%*%t(V_p)
      C <- U%*%diag(D,nrow=rnew)%*%t(V)
      diff <- sqrt(sum((C-C_p)^2)/sum(C_p^2))
        
      
      niter <- niter + 1  
   }
      
   list(U=U,V=V,D=D,niter=niter,conv=diff)
   
}





