\name{RRRR-package}
\alias{RRRR-package}
\alias{RRRR}
\docType{package}
\title{
Regularized Reduced-rank Regression
}
\description{
Regularized reduced-rank regression with sparse singular value decomposition
}
\details{
\tabular{ll}{
Package: \tab RRRR\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2012-08-29\cr
License: \tab GPL-2 \cr
}
%%~~ An overview of how to use the package, including the most important ~~
%%~~ functions ~~
RRR, SRRR, SRRR_eea, SRRR_orth_eea, SRRR_orth_ieea
}
\author{
Kun Chen

Maintainer: Kun Chen <kuniowa@gmail.com>

}
\references{
%%~~ Literature or other references for background information ~~
}
%%~~ Optionally other standard keywords, one per line, from file KEYWORDS in ~~
%%~~ the R documentation directory ~~
\keyword{ package }
\seealso{
%%~~ Optional links to other man pages, e.g. ~~
%%~~ \code{\link[<pkg>:<pkg>-package]{<pkg>}} ~~
}
\examples{

###Simulation model
p <- 25
n <- 50
q <- 25
rho_X <- 0.5
s2n <- 0.5
U <- matrix(ncol=3,nrow=p)
V <- matrix(ncol=3,nrow=q)
V[,1]<-c(sample(c(-1,1),10,replace=TRUE),rep(0,15))
V[,2]<-c(rep(0,12),sample(c(-1,1),10,replace=TRUE),rep(0,3))
V[,3]<-c(rep(0,6),V[7:8,1],-V[9:10,1],sample(c(-1,1),2,replace=TRUE),-V[13:14,2],
        V[15:16,2],rep(0,9))
V[,1] <- V[,1]/lnorm(V[,1],2)
V[,2] <- V[,2]/lnorm(V[,2],2)
V[,3] <- V[,3]/lnorm(V[,3],2)

U[,1]<-c(sample(c(1,-1),5,replace=TRUE)*runif(5,0.3,1),rep(0,20))
U[,2]<-c(rep(0,5),sample(c(1,-1),5,replace=TRUE)*runif(5,0.3,1),rep(0,15))
U[,3]<-c(rep(0,10),sample(c(1,-1),5,replace=TRUE)*runif(5,0.3,1),rep(0,10))  
U[,1] <- U[,1]/lnorm(U[,1],2)
U[,2] <- U[,2]/lnorm(U[,2],2)
U[,3] <- U[,3]/lnorm(U[,3],2)

D <- diag(c(20,10,5))

Xsigma <- matrix(nrow=p,ncol=p,NA)
for(i in 1:p){
    for(j in 1:p){
        Xsigma[i,j] <- rho_X^(abs(i-j))
    }
}
    
X <- mvrnorm(n,rep(0,p),Xsigma)
E <- matrix(nrow=n,ncol=q,rnorm(n*q,0,1))
C <- U\%*\%D\%*\%t(V)
C3 <- U[,3]\%*\%t(V[,3])*D[3,3]
Y3 <- X\%*\%C3
sigma <- sqrt(var(as.numeric(Y3))/var(as.numeric(E))/s2n)    
E <- E*sigma
Y<-X\%*\%C+E

##Use EEA
fit1 <- SRRR_eea(Y,X,nrank=3)
fit1$U
fit1$V
fit1$D

##Use iterative EEA to improve performance
iter <- 8
nrank <- 3
ini <- SRRR_ini(Y,X,nrank=nrank)
ranknow <- nrank
ii <- 1
while(ii <= iter){
  fit <- SRRR_eea(Y,X,nrank=ranknow,ini=ini,pcd.iter=20)
  dorder <- order(fit$D,decreasing=TRUE)
  Ufit <- fit$U[,dorder]
  Vfit <- fit$V[,dorder]
  Dfit <- diag(fit$D[dorder],nrow=length(dorder))
  
  ranknew <- sum(fit$D!=0)
  
  if(ranknew!=1){
    ##update weights if rank changes
    if(ranknow!=ranknew) ini <- SRRR_ini(Y,X,nrank=ranknew)
    ini <- SRRR_ini(Y,X,nrank=ranknew,U0=fit$U[,1:ranknew],V0=fit$V[,1:ranknew],
            D0=fit$D[1:ranknew],Uw=ini$U0[,1:ranknew],Vw=ini$V0[,1:ranknew])
    print(paste("Method=IEEA",ii,"  Rank=",ranknew,sep=""))
    ranknow <- ranknew
    ii <- ii+1
    
  }else{
    ii <- iter+1 
  }
  
}
  

Ufit
Vfit
Dfit



###Use EEA
fit2 <- SRRR_eea(Y,X,nrank=3)
fit2$U
fit2$V
fit2$D



}




