\name{SRRR_eea}
\alias{SRRR_eea}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Sparse Reduced-Rank Regression via Exclusive Extraction Algorithm
}
\description{
Using an exclusive extraction algorithm to perform sparse SVD estimation in reduced-rank regression.
}
\usage{
SRRR_eea(Y, X, nrank, adaptive = TRUE, gamma0 = 2, 
ini = SRRR_ini(Y, X, nrank = nrank, adaptive = adaptive, gamma0 = gamma0), 
nlam = 100, minlam = 1e-05, inner.iter = 100, inner.conv = 1e-04, 
pcd.iter = 50, pcd.conv = 1e-04, IC = 1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Y }{
  An n times q response matrix.
%%     ~~Describe \code{Y} here~~
}
  \item{X }{
  An n times p design matrix.
%%     ~~Describe \code{X} here~~
}
  \item{nrank }{
  Specified rank.
%%     ~~Describe \code{nrank} here~~
}
  \item{adaptive }{
  If TRUE, use adaptive estimation.
%%     ~~Describe \code{adaptive} here~~
}
  \item{gamma0 }{
  The power parameter in the adaptive weights.
%%     ~~Describe \code{gamma0} here~~
}
  \item{ini }{
  Initialization of EEA
%%     ~~Describe \code{ini} here~~
}
  \item{nlam }{
  Number of lambda values
%%     ~~Describe \code{nlam} here~~
}
  \item{minlam }{
  The smallest nonzero lambda value
%%     ~~Describe \code{minlam} here~~
}
  \item{inner.iter }{
  Maximum number of interation allowed in internal sparse unit-rank regression.
%%     ~~Describe \code{inner.iter} here~~
}
  \item{inner.conv }{
  Convergence threshold in internal sparse unit-rank regression.
%%     ~~Describe \code{inner.conv} here~~
}
  \item{pcd.iter }{
  Maximum number of interation allowed in internal cooridinate descent algorithm for lasso.  
%%     ~~Describe \code{pcd.iter} here~~
}
  \item{pcd.conv }{
  Convergence threshold in internal cooridinate descent algorithm for lasso.     
%%     ~~Describe \code{pcd.conv} here~~
}
  \item{IC }{
  Information criteria. IC=1, BIC; IC=2, BICP; IC=3, AIC.
%%     ~~Describe \code{IC} here~~
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
  \item{U_path }{Solution path of U}
  \item{V_path }{Solution path of V}
  \item{D_path }{Solution path of D}
  \item{IC_path }{Information criteria}
  \item{U }{Estiamted U}
  \item{V }{Estiamted V}
  \item{D }{Estiamted D}      
%% ...
}
\references{
%% ~put references to the literature/web site here ~
}
\author{
Kun Chen
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{

###Simulation model
p <- 25
n <- 50
q <- 25
rho_X <- 0.5
s2n <- 0.5
U <- matrix(ncol=3,nrow=p)
V <- matrix(ncol=3,nrow=q)
V[,1]<-c(sample(c(-1,1),10,replace=TRUE),rep(0,15))
V[,2]<-c(rep(0,12),sample(c(-1,1),10,replace=TRUE),rep(0,3))
V[,3]<-c(rep(0,6),V[7:8,1],-V[9:10,1],sample(c(-1,1),2,replace=TRUE),
        -V[13:14,2],V[15:16,2],rep(0,9))
V[,1] <- V[,1]/lnorm(V[,1],2)
V[,2] <- V[,2]/lnorm(V[,2],2)
V[,3] <- V[,3]/lnorm(V[,3],2)

U[,1]<-c(sample(c(1,-1),5,replace=TRUE)*runif(5,0.3,1),rep(0,20))
U[,2]<-c(rep(0,5),sample(c(1,-1),5,replace=TRUE)*runif(5,0.3,1),rep(0,15))
U[,3]<-c(rep(0,10),sample(c(1,-1),5,replace=TRUE)*runif(5,0.3,1),rep(0,10))  
U[,1] <- U[,1]/lnorm(U[,1],2)
U[,2] <- U[,2]/lnorm(U[,2],2)
U[,3] <- U[,3]/lnorm(U[,3],2)

D <- diag(c(20,10,5))

Xsigma <- matrix(nrow=p,ncol=p,NA)
for(i in 1:p){
    for(j in 1:p){
        Xsigma[i,j] <- rho_X^(abs(i-j))
    }
}
    
X <- mvrnorm(n,rep(0,p),Xsigma)
E <- matrix(nrow=n,ncol=q,rnorm(n*q,0,1))
C <- U\%*\%D\%*\%t(V)
C3 <- U[,3]\%*\%t(V[,3])*D[3,3]
Y3 <- X\%*\%C3
sigma <- sqrt(var(as.numeric(Y3))/var(as.numeric(E))/s2n)    
E <- E*sigma
Y<-X\%*\%C+E

##Use EEA
fit <- SRRR_eea(Y,X,nrank=3)
fit$U
fit$V
fit$D

##Use iterative EEA to improve performance
iter <- 8
nrank <- 3
ini <- SRRR_ini(Y,X,nrank=nrank)
ranknow <- nrank
ii <- 1
while(ii <= iter){
  fit <- SRRR_eea(Y,X,nrank=ranknow,ini=ini,pcd.iter=20)
  dorder <- order(fit$D,decreasing=TRUE)
  Ufit <- fit$U[,dorder]
  Vfit <- fit$V[,dorder]
  Dfit <- diag(fit$D[dorder],nrow=length(dorder))
  
  ranknew <- sum(fit$D!=0)
  
  if(ranknew!=1){
    ##update weights if rank changes
    if(ranknow!=ranknew) ini <- SRRR_ini(Y,X,nrank=ranknew)
    ini <- SRRR_ini(Y,X,nrank=ranknew,U0=fit$U[,1:ranknew],V0=fit$V[,1:ranknew],
            D0=fit$D[1:ranknew],Uw=ini$U0[,1:ranknew],Vw=ini$V0[,1:ranknew])
    print(paste("Method=IEEA",ii,"  Rank=",ranknew,sep=""))
    ranknow <- ranknew
    ii <- ii+1
    
  }else{
    ii <- iter+1 
  }
  
}
  

Ufit
Vfit
Dfit





}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{regression}
%%\keyword{sparse}% __ONLY ONE__ keyword per line
